import { Test, TestingModule } from '@nestjs/testing';
import { CreateAdUseCase } from '../../domain/usecases/create-ad.usecase';
import { CreateAdRequest } from '../../domain/dtos/create-ad.request';
import { Address } from '../../domain/entities/address';
import { Messager } from '../../adapters/secondaries/messager';
import { AdsRepository } from '../../adapters/secondaries/ads.repository';
import { CreateAdCommand } from '../../commands/create-ad.command';
import { AddressType } from '../../domain/entities/addressEnum';
import { AutomapperModule } from '@automapper/nestjs';
import { classes } from '@automapper/classes';
import { FrequencyType } from '../../domain/entities/frequencyEnum';
import { Ad } from '../../domain/entities/ad';
import { AdProfile } from '../../mappers/ad.profile';

const mockAddress1: Address = {
  position: 0,
  lon: 48.68944505415954,
  lat: 6.176510296462267,
  houseNumber: '5',
  street: 'Avenue Foch',
  locality: 'Nancy',
  postalCode: '75000',
  country: 'France',
  type: AddressType.HOUSE_NUMBER,
};
const mockAddress2: Address = {
  position: 1,
  lon: 48.8566,
  lat: 2.3522,
  locality: 'Paris',
  postalCode: '75000',
  country: 'France',
  type: AddressType.LOCALITY,
};

const newAdRequest: CreateAdRequest = {
  userUuid: '113e0000-0000-4000-a000-000000000000',
  driver: true,
  passenger: false,
  frequency: FrequencyType.RECURRENT,
  fromDate: new Date('01-05-2023'),
  toDate: new Date('20-08-2023'),
  tueTime: new Date(''),
  wedTime: new Date(''),
  seatsPassenger: 2,
  addresses: [mockAddress1, mockAddress2],
};

const newAdCommand = new CreateAdCommand(newAdRequest);

const mockMessager = {
  publish: jest.fn().mockImplementation(),
};
const mockAdRepository = {
  create: jest
    .fn()
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    .mockImplementationOnce((command?: CreateAdCommand) => {
      return Promise.resolve({
        ...newAdRequest,
        uuid: 'ad000000-0000-4000-a000-000000000000',
        createdAt: new Date('01-05-2023'),
      });
    })
    .mockImplementation(() => {
      throw new Error('Already exists');
    }),
};

describe('CreateAdUseCase', () => {
  let createAdUseCase: CreateAdUseCase;
  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [AutomapperModule.forRoot({ strategyInitializer: classes() })],
      providers: [
        {
          provide: AdsRepository,
          useValue: mockAdRepository,
        },
        {
          provide: Messager,
          useValue: mockMessager,
        },
        CreateAdUseCase,
        AdProfile,
      ],
    }).compile();

    createAdUseCase = module.get<CreateAdUseCase>(CreateAdUseCase);
  });
  it('should be defined', () => {
    expect(createAdUseCase).toBeDefined();
  });

  describe('execution', () => {
    it('should create an new ad', async () => {
      const newAd: Ad = await createAdUseCase.execute(newAdCommand);
      expect(newAd.userUuid).toBe(newAdRequest.userUuid);
      expect(newAd.uuid).toBeDefined();
    });
    it('should throw an error if the ad already exists', async () => {
      await expect(
        createAdUseCase.execute(newAdCommand),
      ).rejects.toBeInstanceOf(Error);
    });
  });
});
